# stats.tcl
#	system statistics frame 


set debug 0
set started 0

set frame STATS
set frametitle "System Statistics"
set icontitle  "SysStats"

set RDOUTserverList [inform sys inquire RDOUTserverList]
set RDOUTserver [first $RDOUTserverList]

set RefreshPeriod 15


set refreshing 0
set afterid ""

#     the following labels and custom procedures for overriding in application specific code
set NUM 32
for {set i 0} {$i <= 11} {incr i} {set ITEM($i) "$i"}

proc custom_pre_make {} {}
proc custom_post_make {} {}
proc custom_clear {} {}
proc custom_redisplay {} {}

proc NEWread-Rate {n} {global Rates; return $Rates($n)}

proc read-Stat {i} {
  global RDOUTserver RDOUThdwre

	set reg /$RDOUTserver/NetVar.DACQ.Stats
        EG "register :name '$reg' :object '-i $i'"
	return [EG "register :name '$reg' ?value"]
}

proc read-Rate {i} {
  global RDOUTserver RDOUThdwre

	set reg /$RDOUTserver/NetVar.DACQ.Rates
        EG "register :name '$reg' :object '-i $i'"
	return [EG "register :name '$reg' ?value"]
}

proc NEWclick-redisplay {} {
   global RDOUTserver NUM ITEM Stats Rates
   global EGreport

   set-footer "Updating..."
   set EGreport -1
   set-busy

   set StatsBlock [EG "register :name '/$RDOUTserver/NetVar.DACQ.Stats1' ?value"]
   append StatsBlock [EG "register :name '/$RDOUTserver/NetVar.DACQ.Stats2' ?value"]

   set RatesBlock [EG "register :name '/$RDOUTserver/NetVar.DACQ.Rates1' ?value"]
   append RatesBlock [EG "register :name '/$RDOUTserver/NetVar.DACQ.Rates1' ?value"]

   for {set i 0} {$i < $NUM} {incr i} {
       set Stats($i) [lindex $StatsBlock [expr $i*2]]
       if {[info exists ITEM($i)]} then {
	   :modify STAT$i :value [expr $Stats($i)]
       }
   }

   for {set i 0} {$i < $NUM} {incr i} {
       set Rates($i) [lindex $RatesBlock [expr $i*2]]
       if {[info exists ITEM($i)]} then {
           :modify RATE$i :value [expr $Rates($i)]
       }
   }

   custom_redisplay

   clear-busy
   set EGreport 0
   set-footer "Counts Updated [date]"
}

proc click-redisplay {} {
   global RDOUTserver NUM ITEM Stats Rates
   global EGreport

   set-footer "Updating..."
   set EGreport -1
   set-busy

   for {set i 0} {$i < $NUM} {incr i} {
       if {[info exists ITEM($i)]} then {
           set Stats($i) [read-Stat $i]
	   :modify STAT$i :value $Stats($i)
       }
   }

   for {set i 0} {$i < $NUM} {incr i} {
       if {[info exists ITEM($i)]} then {
            set Rates($i) [read-Rate $i]
           :modify RATE$i :value $Rates($i)
       }
   }

   custom_redisplay

   clear-busy
   set EGreport 0
   set-footer "Counts Updated [date]"
}


proc click-clear {} {

   inform sys do_clearstats
   custom_clear

   click-redisplay
}

proc click-refresh {val} {
	global debug afterid refreshing
	set refreshing $val
	if {$debug} then {puts stderr "click-refresh $val"}
	set afterid ""
	if {$val!=0} then {set afterid "xxx"; do_refresh}
}

proc do_refresh {} {
   global afterid RefreshPeriod
   catch click-redisplay
   if {$afterid!=""} then {after [expr $RefreshPeriod * 1000] do_refresh}
}

proc click arg {
   global debug RDOUTserver RDOUTserverList
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {return [click-quit]}
   switch $w {
      REDISPLAY {click-redisplay}
      REFRESH   {click-refresh $v}
      CLEAR     {click-clear}
      SERVER    {set RDOUTserver [lindex $RDOUTserverList $v]; click-redisplay}
   }
}

proc start args {
   global started refreshing
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   if {$refreshing} then {click-refresh 1} else {click-redisplay}
   :modify REFRESH :value $refreshing
}

proc click-quit {} {
   click-refresh 0
   close-frame
}

proc finish args {destroy-frame}


proc make-frame {} {
  global env refreshing NUM ITEM RDOUTserverList

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]" :resize F

      custom_pre_make

   :panel COMMON :layout T :padx 10 :pady 10
     :layout H; :col-gap 10

  if {[llength $RDOUTserverList] > 1} then {
     :choice SERVER :label "Data Acquisition server" :strings $RDOUTserverList :exclusive T :value 0
     :row-gap 10; :next-row T; :row-gap 0
  }

       :message CMC :label Counters :bold T
       :next-row T
       for {set i 0; set j 0} {$j < $NUM} {incr j} {
          if {[info exists ITEM($j)]} then {
              :text STAT$j :label "$ITEM($j)" :w 10 :read-only T :valuex 24c
              incr i
              if {[expr ($i/3)*3] == $i} {:next-row T}
          }
       }

       :next-row T
       :message CMR :label "Rates (/sec)" :bold T
       :next-row T
       for {set i 0; set j 0} {$j < $NUM} {incr j} {
          if {[info exists ITEM($j)]} then {
              :text RATE$j :label "$ITEM($j)" :w 10 :read-only T :valuex 24c
              incr i
              if {[expr ($i/3)*3] == $i} {:next-row T}
          }
       }

      custom_post_make

     :panel BUTTONS :layout B :padx 10
       :layout H; :col-gap 40
       :button REDISPLAY
       :choice REFRESH   :label Refresh: :strings [list Off On] :value $refreshing
       :button CLEAR
       :button HELP     :command click-help

        :show T
}


#    support for Web Service access

#    the order of counters and rates returned is

#    1  good events   (counter 0)
#    2  bad events    (counter 11)
#    3  input data buffers  (for non buffered input return 0)
#    4  output data buffers  (counter 4)

proc Get_Counters {} {

    set m1 [read-Stat 0]
    set m2 [read-Stat 11]
    set m3 "0"
    set m4 [read-Stat 4]

    return [list $m1 $m2 $m3 $m4]
}

proc Get_Rates {} {

    set m1 [read-Rate 0]
    set m2 [read-Rate 11]
    set m3 "0"
    set m4 [read-Rate 4]

    return [list $m1 $m2 $m3 $m4]
}
